const status = async (...args) => {
  const [pool, readModelName, eventstoreAdapter, includeRuntimeStatus] = args;

  try {
    var _result;

    pool.activePassthrough = true;
    const {
      PassthroughError,
      schemaName,
      escapeId,
      escapeStr,
      inlineLedgerExecuteStatement
    } = pool;
    let result = null;
    let cursorAndEventTypes = [null, ['*']];
    const databaseNameAsId = escapeId(schemaName);
    const ledgerTableNameAsId = escapeId(`__${schemaName}__LEDGER__`);
    const trxTableNameAsId = escapeId(`__${schemaName}__TRX__`);
    const rows = await inlineLedgerExecuteStatement(pool, `SELECT * FROM ${databaseNameAsId}.${ledgerTableNameAsId}
     WHERE "EventSubscriber" = ${escapeStr(readModelName)}
    `);

    if (rows.length === 1) {
      result = {
        eventSubscriber: readModelName,
        deliveryStrategy: 'inline-ledger',
        successEvent: rows[0].SuccessEvent != null ? JSON.parse(rows[0].SuccessEvent) : null,
        failedEvent: rows[0].FailedEvent != null ? JSON.parse(rows[0].FailedEvent) : null,
        errors: rows[0].Errors != null ? JSON.parse(rows[0].Errors) : null,
        cursor: rows[0].Cursor != null ? JSON.parse(rows[0].Cursor) : null,
        status: 'deliver'
      };

      if (result.errors != null) {
        result.status = 'error';
      } else if (rows[0].IsPaused) {
        result.status = 'skip';
      }

      cursorAndEventTypes = [result.cursor, rows[0].EventTypes != null ? JSON.parse(rows[0].EventTypes) : ['*']];
    }

    if (includeRuntimeStatus && ((_result = result) === null || _result === void 0 ? void 0 : _result.status) === 'deliver') {
      var _eventstoreAdapter$ge, _eventstoreAdapter$ge2;

      let isActive = false;
      const endTime = Date.now() + 5000;

      for (let currentTime = 0; currentTime < endTime; currentTime = Date.now()) {
        try {
          var _rows$;

          const rows = await inlineLedgerExecuteStatement(pool, `SELECT "T"."XaValue" AS "TransactionId"
              FROM ${databaseNameAsId}.${ledgerTableNameAsId} "L"
              LEFT JOIN ${databaseNameAsId}.${trxTableNameAsId} "T" ON "L"."XaKey" = "T"."XaKey"
              WHERE "L"."EventSubscriber" = ${escapeStr(readModelName)}
              AND "L"."IsPaused" = FALSE
              AND "L"."Errors" IS NULL
              LIMIT 1;
            `);

          if ((rows === null || rows === void 0 ? void 0 : (_rows$ = rows[0]) === null || _rows$ === void 0 ? void 0 : _rows$.TransactionId) != null) {
            try {
              var _rows$2;

              await inlineLedgerExecuteStatement(pool, 'SELECT 0', rows === null || rows === void 0 ? void 0 : (_rows$2 = rows[0]) === null || _rows$2 === void 0 ? void 0 : _rows$2.TransactionId);
              isActive = true;
            } catch (e) {}
          }
        } catch (error) {
          if (!(error instanceof PassthroughError) || !error.isRetryable) {
            throw error;
          }
        }
      }

      const [nextCursor, endCursor] = await Promise.all([(_eventstoreAdapter$ge = eventstoreAdapter.getCursorUntilEventTypes) === null || _eventstoreAdapter$ge === void 0 ? void 0 : _eventstoreAdapter$ge.call(eventstoreAdapter, ...cursorAndEventTypes), (_eventstoreAdapter$ge2 = eventstoreAdapter.getCursorUntilEventTypes) === null || _eventstoreAdapter$ge2 === void 0 ? void 0 : _eventstoreAdapter$ge2.call(eventstoreAdapter, cursorAndEventTypes[0], ['*'])]);
      const hasNextEvents = nextCursor != null && nextCursor !== endCursor;
      const isAlive = isActive || !hasNextEvents;
      result = Object.assign(result, {
        isAlive
      });
    } else if (includeRuntimeStatus) {
      var _result2;

      const isAlive = ((_result2 = result) === null || _result2 === void 0 ? void 0 : _result2.status) === 'skip';
      result = Object.assign(result, {
        isAlive
      });
    }

    return result;
  } finally {
    pool.activePassthrough = false;
  }
};

export default status;
//# sourceMappingURL=status.js.map